/* eslint-disable no-magic-numbers */
import { PINS_COLORS, TEXT_COLOR } from '@/constants/canvas';
import { bioEntityContentFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { initialMapStateFixture } from '@/redux/map/map.fixtures';
import { PinType } from '@/types/pin';
import { addAlphaToHexString } from '@/utils/convert/addAlphaToHexString';
import { UsePointToProjectionResult, usePointToProjection } from '@/utils/map/usePointToProjection';
import {
  GetReduxWrapperUsingSliceReducer,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { renderHook } from '@testing-library/react';
import { Feature } from 'ol';
import Style from 'ol/style/Style';
import { getBioEntitySingleFeature } from './getBioEntitySingleFeature';
import * as getPinStyle from './getPinStyle';

jest.mock('./getPinStyle', () => ({
  __esModule: true,
  ...jest.requireActual('./getPinStyle'),
}));

const getPointToProjection = (
  wrapper: ReturnType<GetReduxWrapperUsingSliceReducer>['Wrapper'],
): UsePointToProjectionResult => {
  const { result: usePointToProjectionHook } = renderHook(() => usePointToProjection(), {
    wrapper,
  });

  return usePointToProjectionHook.current;
};

describe('getBioEntitySingleFeature - subUtil', () => {
  const { Wrapper } = getReduxWrapperWithStore({
    map: initialMapStateFixture,
  });
  const { bioEntity } = bioEntityContentFixture;
  const pointToProjection = getPointToProjection(Wrapper);

  const value = 1448;
  const pinTypes: PinType[] = ['bioEntity', 'drugs', 'chemicals'];

  it.each(pinTypes)('should return instance of Feature with Style type=%s', type => {
    const result = getBioEntitySingleFeature(bioEntity, {
      pointToProjection,
      type,
      value,
      isActive: true,
    });

    const style = result.getStyle() as Style;

    expect(result).toBeInstanceOf(Feature);
    expect(style).toBeInstanceOf(Style);
  });

  it.each(pinTypes)('should run getPinStyle with valid args for type=%s', type => {
    const getPinStyleSpy = jest.spyOn(getPinStyle, 'getPinStyle');

    getBioEntitySingleFeature(bioEntity, {
      pointToProjection,
      type,
      value,
      isActive: true,
    });

    expect(getPinStyleSpy).toHaveBeenCalledWith({
      color: PINS_COLORS[type],
      value,
      textColor: TEXT_COLOR,
    });
  });

  it.each(pinTypes)(
    'should run getPinStyle with valid args for isActive=fasle and type=%s',
    type => {
      const getPinStyleSpy = jest.spyOn(getPinStyle, 'getPinStyle');

      getBioEntitySingleFeature(bioEntity, {
        pointToProjection,
        type,
        value,
        isActive: false,
      });

      expect(getPinStyleSpy).toHaveBeenCalledWith({
        color: addAlphaToHexString(PINS_COLORS[type], 0.5),
        value,
        textColor: addAlphaToHexString(TEXT_COLOR, 0.5),
      });
    },
  );
});
