/* eslint-disable no-magic-numbers */
import { Point as PointType } from '@/types/map';
import { Reaction } from '@/types/models';
import { distance } from 'ol/coordinate';
import { LineString, Point } from 'ol/geom';
import { getMaxClickDistance } from './getMaxClickDistance';

type ReactionLine = Reaction['lines'][0];

type FindClosestReactionArgs = {
  reaction: Reaction;
  searchDistance: string;
  maxZoom: number;
  zoom: number;
  point: PointType;
};

export const findClosestReactionPoint = ({
  reaction,
  searchDistance,
  maxZoom,
  zoom,
  point,
}: FindClosestReactionArgs): ReactionLine | undefined => {
  const maxDistance = getMaxClickDistance(maxZoom, zoom, searchDistance);

  const clickedPoint = new Point([point.x, point.y]);

  const closestLine = reaction.lines.find(line => {
    const lineString = new LineString([
      [line.start.x, line.start.y],
      [line.end.x, line.end.y],
    ]);

    const closestPointOnLine = lineString.getClosestPoint(clickedPoint.getCoordinates());
    const distanceToLine = distance(closestPointOnLine, clickedPoint.getCoordinates());

    return distanceToLine <= maxDistance;
  });

  return closestLine;
};
