/* eslint-disable @typescript-eslint/explicit-function-return-type */
import { renderHook } from '@testing-library/react';
import { View } from 'ol';
import { getReduxWrapperWithStore } from '@/utils/testing/getReduxWrapperWithStore';
import { initialMapDataFixture, openedMapsThreeSubmapsFixture } from '@/redux/map/map.fixtures';
import * as singleClickListener from './mapSingleClick/onMapSingleClick';
import * as positionListener from './onMapPositionChange';
import { useOlMapListeners } from './useOlMapListeners';

jest.mock('./onMapPositionChange', () => ({
  __esModule: true,
  onMapPositionChange: jest.fn(),
}));

jest.mock('./mapSingleClick/onMapSingleClick', () => ({
  __esModule: true,
  onMapSingleClick: jest.fn(),
}));

jest.mock('use-debounce', () => {
  return {
    useDebounce: () => {},
    useDebouncedCallback: () => {},
  };
});

describe('useOlMapListeners - util', () => {
  const { Wrapper } = getReduxWrapperWithStore({
    map: {
      data: { ...initialMapDataFixture },
      loading: 'succeeded',
      error: { message: '', name: '' },
      openedMaps: openedMapsThreeSubmapsFixture,
    },
  });

  beforeEach(() => {
    jest.clearAllMocks();
  });

  describe('on change:center view event', () => {
    it('should run onMapPositionChange event', () => {
      const CALLED_ONCE = 1;
      const view = new View();

      renderHook(() => useOlMapListeners({ view, mapInstance: undefined }), { wrapper: Wrapper });
      view.dispatchEvent('change:center');

      expect(positionListener.onMapPositionChange).toBeCalledTimes(CALLED_ONCE);
    });
  });

  describe('on singleclick view event', () => {
    it('should run onMapPositionChange event', () => {
      const CALLED_ONCE = 1;
      const view = new View();

      renderHook(() => useOlMapListeners({ view, mapInstance: undefined }), { wrapper: Wrapper });
      view.dispatchEvent('singleclick');

      expect(singleClickListener.onMapSingleClick).toBeCalledTimes(CALLED_ONCE);
    });
  });
});
